@@ -1304,7 +1304,10 @@ REGISTER_TYPED_TEST_SUITE_P(
 
 using folly::detail::concurrenthashmap::bucket::BucketTable;
 
-#if FOLLY_SSE_PREREQ(4, 2) && FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
+#if (                                                        \
+    FOLLY_SSE_PREREQ(4, 2) ||                                \
+    (FOLLY_AARCH64 && FOLLY_F14_CRC_INTRINSIC_AVAILABLE)) && \
+    FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 using folly::detail::concurrenthashmap::simd::SIMDTable;
 using MapFactoryTypes =
     ::testing::Types<MapFactory<BucketTable>, MapFactory<SIMDTable>>;
