@@ -82,9 +82,7 @@ class TaskPromiseBase {
     bool await_ready() noexcept { return false; }
 
     template <typename Promise>
-    FOLLY_CORO_AWAIT_SUSPEND_NONTRIVIAL_ATTRIBUTES coroutine_handle<>
-    await_suspend(coroutine_handle<Promise> coro) noexcept {
-      auto& promise = coro.promise();
+    coroutine_handle<> await_suspend_promise(Promise& promise) noexcept {
       // If ScopeExitTask has been attached, then we expect that the
       // ScopeExitTask will handle the lifetime of the async stack. See
       // ScopeExitTaskPromise's FinalAwaiter for more details.
@@ -114,6 +112,12 @@ class TaskPromiseBase {
       return promise.continuationRef(privateTag()).getHandle();
     }
 
+    template <typename Promise>
+    FOLLY_CORO_AWAIT_SUSPEND_NONTRIVIAL_ATTRIBUTES coroutine_handle<>
+    await_suspend(coroutine_handle<Promise> coro) noexcept {
+      return await_suspend_promise(coro.promise());
+    }
+
     [[noreturn]] void await_resume() noexcept { folly::assume_unreachable(); }
   };
 
@@ -178,16 +182,17 @@ class TaskPromiseBase {
 
   template <typename Awaitable>
   auto await_transform(NothrowAwaitable<Awaitable> awaitable) {
+    static_assert(!noexcept_awaitable_v<Awaitable>); // Doc on NothrowAwaitable
     bypassExceptionThrowing_ = BypassExceptionThrowing::REQUESTED;
     return await_transform(
         folly::ext::must_use_immediately_unsafe_mover(awaitable.unwrap())());
   }
 
-  auto await_transform(co_current_executor_t) noexcept {
+  auto await_transform(co_current_executor_t /*unused*/) noexcept {
     return ready_awaitable<folly::Executor*>{executor_.get()};
   }
 
-  auto await_transform(co_current_cancellation_token_t) noexcept {
+  auto await_transform(co_current_cancellation_token_t /*unused*/) noexcept {
     return ready_awaitable<const folly::CancellationToken&>{cancelToken_};
   }
 
@@ -207,10 +212,12 @@ class TaskPromiseBase {
   // These getters exist so that `FinalAwaiter` can interact with wrapped
   // `TaskPromise`s, and not just `TaskPromiseBase` descendants.  We use a
   // private tag to let `TaskWrapper` call them without becoming a `friend`.
-  auto& scopeExitRef(TaskPromisePrivate) { return scopeExit_; }
-  auto& continuationRef(TaskPromisePrivate) { return continuation_; }
+  auto& scopeExitRef(TaskPromisePrivate /*unused*/) { return scopeExit_; }
+  // FIXME: `result/coro.h` checks if this overload is callable to decide when
+  // something is a task-promise.  A second use-case would merit a concept.
+  auto& continuationRef(TaskPromisePrivate /*unused*/) { return continuation_; }
   // Unlike `getExecutor()`, does not copy an atomic.
-  auto& executorRef(TaskPromisePrivate) { return executor_; }
+  auto& executorRef(TaskPromisePrivate /*unused*/) { return executor_; }
 
  private:
   template <typename>
@@ -220,7 +227,7 @@ class TaskPromiseBase {
   friend class folly::coro::Task;
 
   friend coroutine_handle<ScopeExitTaskPromiseBase> tag_invoke(
-      cpo_t<co_attachScopeExit>,
+      cpo_t<co_attachScopeExit> /*unused*/,
       TaskPromiseBase& p,
       coroutine_handle<ScopeExitTaskPromiseBase> scopeExit) noexcept {
     return std::exchange(p.scopeExit_, scopeExit);
@@ -245,7 +252,7 @@ class TaskPromiseBase {
 template <typename Promise, typename T>
 class TaskPromiseCrtpBase
     : public TaskPromiseBase,
-      public ExtendedCoroutinePromise {
+      public ExtendedCoroutinePromiseCrtp<Promise> {
  public:
   using StorageType = detail::lift_lvalue_reference_t<T>;
 
@@ -269,29 +276,27 @@ class TaskPromiseCrtpBase
 
   using TaskPromiseBase::await_transform;
 
-  auto await_transform(co_safe_point_t) noexcept {
+  auto await_transform(co_safe_point_t /*unused*/) noexcept {
     return do_safe_point(*this);
   }
 
+  static std::optional<ExtendedCoroutineHandle::ErrorHandle> getErrorHandleImpl(
+      Promise& me, exception_wrapper& ex) {
+    if (me.bypassExceptionThrowing_ == BypassExceptionThrowing::ACTIVE) {
+      auto finalAwaiter = me.yield_value(co_error(std::move(ex)));
+      DCHECK(!finalAwaiter.await_ready());
+      return ExtendedCoroutineHandle::ErrorHandle{
+          finalAwaiter.await_suspend_promise(me),
+          // finalAwaiter.await_suspend pops a frame
+          me.getAsyncFrame().getParentFrame()};
+    }
+    return std::nullopt;
+  }
+
  protected:
   TaskPromiseCrtpBase() noexcept = default;
   ~TaskPromiseCrtpBase() = default;
 
-  std::pair<ExtendedCoroutineHandle, AsyncStackFrame*> getErrorHandle(
-      exception_wrapper& ex) final {
-    auto& me = *static_cast<Promise*>(this);
-    if (bypassExceptionThrowing_ == BypassExceptionThrowing::ACTIVE) {
-      auto finalAwaiter = yield_value(co_error(std::move(ex)));
-      DCHECK(!finalAwaiter.await_ready());
-      return {
-          finalAwaiter.await_suspend(
-              coroutine_handle<Promise>::from_promise(me)),
-          // finalAwaiter.await_suspend pops a frame
-          getAsyncFrame().getParentFrame()};
-    }
-    return {coroutine_handle<Promise>::from_promise(me), nullptr};
-  }
-
   Try<StorageType> result_;
 };
 
@@ -501,7 +506,7 @@ class FOLLY_NODISCARD TaskWithExecutor {
     // it will not have a chance to restore the previous context before we
     // return, so we need to ensure it is restored. This simulates starting the
     // coroutine in an actual executor, which would wrap the task with a guard.
-    RequestContextScopeGuard contextScope{RequestContext::saveContext()};
+    RequestContextSaverScopeGuard contextScope;
     startInlineImpl(std::move(*this), static_cast<F&&>(tryCallback))
         .start(returnAddress);
   }
@@ -670,7 +675,8 @@ class FOLLY_NODISCARD TaskWithExecutor {
 
    private:
     friend InlineTryAwaitable tag_invoke(
-        cpo_t<co_withAsyncStack>, InlineTryAwaitable&& awaitable) noexcept {
+        cpo_t<co_withAsyncStack> /*unused*/,
+        InlineTryAwaitable&& awaitable) noexcept {
       return std::move(awaitable);
     }
 
@@ -708,7 +714,7 @@ class FOLLY_NODISCARD TaskWithExecutor {
   }
 
   friend TaskWithExecutor tag_invoke(
-      cpo_t<co_withAsyncStack>, TaskWithExecutor&& task) noexcept {
+      cpo_t<co_withAsyncStack> /*unused*/, TaskWithExecutor&& task) noexcept {
     return std::move(task);
   }
 
@@ -865,7 +871,10 @@ class FOLLY_CORO_TASK_ATTRS Task {
 
   template <typename F, typename... A, typename F_, typename... A_>
   friend Task tag_invoke(
-      tag_t<co_invoke_fn>, tag_t<Task, F, A...>, F_ f, A_... a) {
+      tag_t<co_invoke_fn> /*unused*/,
+      tag_t<Task, F, A...> /*unused*/,
+      F_ f,
+      A_... a) {
     co_yield co_result(co_await co_awaitTry(
         invoke(static_cast<F&&>(f), static_cast<A&&>(a)...)));
   }
@@ -954,7 +963,7 @@ class FOLLY_CORO_TASK_ATTRS Task {
     // This overload needed as Awaiter is returned from co_viaIfAsync() which is
     // then passed into co_withAsyncStack().
     friend Awaiter tag_invoke(
-        cpo_t<co_withAsyncStack>, Awaiter&& awaiter) noexcept {
+        cpo_t<co_withAsyncStack> /*unused*/, Awaiter&& awaiter) noexcept {
       return std::move(awaiter);
     }
 
@@ -978,7 +987,7 @@ inline Task<void> makeTask() {
   co_return;
 }
 /// Same as makeTask(). See Unit
-inline Task<void> makeTask(Unit) {
+inline Task<void> makeTask(Unit /*unused*/) {
   co_return;
 }
 
@@ -1001,6 +1010,10 @@ Task<drop_unit_t<T>> makeResultTask(Try<T> t) {
 template <typename Promise, typename T>
 inline Task<T>
 detail::TaskPromiseCrtpBase<Promise, T>::get_return_object() noexcept {
+  // Watch out: When used with `TaskWrapper`, this relies on "practically safe"
+  // UB wherein this handle is only valid because `TaskPromise` and the true
+  // "wrapper promise" of the wrapper coro coincide in layout exactly.
+  // Documented in `TaskPromiseWrapperBase::is_promise_type_punning_safe`.
   return Task<T>{
       coroutine_handle<Promise>::from_promise(*static_cast<Promise*>(this))};
 }
