@@ -21,13 +21,13 @@
 
 #include <folly/Optional.h>
 #include <folly/Range.h>
-#include <folly/experimental/symbolizer/Elf.h>
+#include <folly/debugging/symbolizer/Elf.h>
 #include <folly/portability/Config.h>
 #include <folly/portability/Unistd.h>
 
 #if FOLLY_HAVE_DWARF && FOLLY_HAVE_ELF
 
-#include <dwarf.h> // @manual=fbsource//third-party/libdwarf:dwarf
+#include <dwarf.h> // @manual
 
 // We need a single dwarf5 tag, but may not be building against
 // a new enough libdwarf, so just define it ourselves.
@@ -761,7 +761,6 @@ Attribute readAttribute(
       FOLLY_SAFE_DFATAL("invalid attribute form: ", spec.form);
       return {spec, die, 0u};
   }
-  return {spec, die, 0u};
 }
 
 /*
