@@ -25,7 +25,6 @@
 #include <string>
 #include <thread>
 #include <unordered_map>
-#include <unordered_set>
 #include <vector>
 
 #include <glog/logging.h>
@@ -265,60 +264,6 @@ struct ThreadEntryList {
   size_t count{0};
 };
 
-/**
- * We want to disable onThreadExit call at the end of shutdown, we don't care
- * about leaking memory at that point.
- *
- * Otherwise if ThreadLocal is used in a shared library, onThreadExit may be
- * called after dlclose().
- *
- * This class has one single static instance; however since it's so widely used,
- * directly or indirectly, by so many classes, we need to take care to avoid
- * problems stemming from the Static Initialization/Destruction Order Fiascos.
- * Therefore this class needs to be constexpr-constructible, so as to avoid
- * the need for this to participate in init/destruction order.
- */
-class PthreadKeyUnregister {
- public:
-  static constexpr size_t kMaxKeys = size_t(1) << 16;
-
-  ~PthreadKeyUnregister() {
-    // If static constructor priorities are not supported then
-    // ~PthreadKeyUnregister logic is not safe.
-#if !defined(__APPLE__) && !defined(_MSC_VER)
-    MSLGuard lg(lock_);
-    while (size_) {
-      pthread_key_delete(keys_[--size_]);
-    }
-#endif
-  }
-
-  static void registerKey(pthread_key_t key) { instance_.registerKeyImpl(key); }
-
- private:
-  /**
-   * Only one global instance should exist, hence this is private.
-   * See also the important note at the top of this class about `constexpr`
-   * usage.
-   */
-  constexpr PthreadKeyUnregister() : lock_(), size_(0), keys_() {}
-
-  void registerKeyImpl(pthread_key_t key) {
-    MSLGuard lg(lock_);
-    if (size_ == kMaxKeys) {
-      throw_exception<std::logic_error>(
-          "pthread_key limit has already been reached");
-    }
-    keys_[size_++] = key;
-  }
-
-  MicroSpinLock lock_;
-  size_t size_;
-  pthread_key_t keys_[kMaxKeys];
-
-  static PthreadKeyUnregister instance_;
-};
-
 /**
  * Cache the ptr + deleter info in ThreadEntrySet too. This allows
  * accessAllThreads() to get to the per thread ptr without holding the
@@ -350,105 +295,31 @@ struct ThreadEntrySet {
 
   bool basicSanity() const;
 
-  void clear() {
-    DCHECK(basicSanity());
-    entryToVectorSlot.clear();
-    threadElements.clear();
-  }
+  void clear();
 
-  int64_t getIndexFor(ThreadEntry* entry) const {
-    auto iter = entryToVectorSlot.find(entry);
-    if (iter != entryToVectorSlot.end()) {
-      return static_cast<int64_t>(iter->second);
-    }
-    return -1;
-  }
+  int64_t getIndexFor(ThreadEntry* entry) const;
 
   /**
    * Helper function for debugging checks. Fetch ptr for a given ThreadEnrtry.
    * Used to sanity check the value in ElementDisposeInfo wrapper matches the
    * ElementWrapper array used for fast access from the thread itself.
    */
-  void* getPtrForThread(ThreadEntry* entry) const {
-    auto index = getIndexFor(entry);
-    if (index < 0) {
-      return nullptr;
-    }
-    return threadElements[static_cast<size_t>(index)].wrapper.ptr;
-  }
+  void* getPtrForThread(ThreadEntry* entry) const;
 
-  bool contains(ThreadEntry* entry) const {
-    DCHECK(basicSanity());
-    return entryToVectorSlot.find(entry) != entryToVectorSlot.end();
-  }
+  bool contains(ThreadEntry* entry) const;
 
-  bool insert(ThreadEntry* entry) {
-    DCHECK(basicSanity());
-    auto iter = entryToVectorSlot.find(entry);
-    if (iter != entryToVectorSlot.end()) {
-      // Entry already present. Sanity check and exit.
-      DCHECK_EQ(entry, threadElements[iter->second].threadEntry);
-      return false;
-    }
-    threadElements.emplace_back(entry);
-    auto idx = threadElements.size() - 1;
-    entryToVectorSlot[entry] = idx;
-    return true;
-  }
+  bool insert(ThreadEntry* entry);
 
-  bool insert(const Element& element) {
-    DCHECK(basicSanity());
-    auto iter = entryToVectorSlot.find(element.threadEntry);
-    if (iter != entryToVectorSlot.end()) {
-      // Entry already present. Skip copying over element. Caller
-      // responsible for handling acceptability of this behavior.
-      DCHECK_EQ(element.threadEntry, threadElements[iter->second].threadEntry);
-      return false;
-    }
-    threadElements.push_back(element);
-    auto idx = threadElements.size() - 1;
-    entryToVectorSlot[element.threadEntry] = idx;
-    return true;
-  }
+  bool insert(const Element& element);
 
-  Element erase(ThreadEntry* entry) {
-    DCHECK(basicSanity());
-    auto iter = entryToVectorSlot.find(entry);
-    if (iter == entryToVectorSlot.end()) {
-      // Entry not present.
-      return Element{nullptr};
-    }
-    auto idx = iter->second;
-    DCHECK_LT(idx, threadElements.size());
-    entryToVectorSlot.erase(iter);
-    Element last = threadElements.back();
-    Element current = threadElements[idx];
-    if (idx != threadElements.size() - 1) {
-      threadElements[idx] = last;
-      entryToVectorSlot[last.threadEntry] = idx;
-    }
-    threadElements.pop_back();
-    DCHECK(basicSanity());
-    if (compressible()) {
-      compress();
-    }
-    DCHECK(basicSanity());
-    return current;
-  }
+  Element erase(ThreadEntry* entry);
 
   /// compressible
   ///
   /// If many elements have been removed, then size might be much less than
   /// capacity and it becomes possible to reduce memory usage.
-  bool compressible() const {
-    // We choose a sufficiently-large multiplier so that there is no risk of a
-    // following insert growing the vector and then a following erase shrinking
-    // the vector, since that way lies non-amortized-O(N)-complexity costs for
-    // both insert and erase ops.
-    constexpr size_t const mult = 4;
-    auto& vec = threadElements;
-    return std::max(size_t(1), vec.size()) * mult <= vec.capacity();
-  }
+  bool compressible() const;
+
   /// compress
   ///
   /// Attempt to reduce the memory usage of the data structure.
@@ -505,6 +376,8 @@ struct StaticMetaBase {
 
   FOLLY_EXPORT static ThreadEntryList* getThreadEntryList();
 
+  ThreadEntry* allocateNewThreadEntry();
+
   static bool dying();
 
   static void onThreadExit(void* ptr);
@@ -796,22 +669,7 @@ struct FOLLY_EXPORT StaticMeta final : StaticMetaBase {
     ThreadEntry* threadEntry =
         static_cast<ThreadEntry*>(pthread_getspecific(key));
     if (!threadEntry) {
-      ThreadEntryList* threadEntryList = StaticMeta::getThreadEntryList();
-      threadEntry = new ThreadEntry();
-
-      threadEntry->list = threadEntryList;
-      threadEntry->listNext = threadEntryList->head;
-      threadEntryList->head = threadEntry;
-
-      threadEntry->tid() = std::this_thread::get_id();
-      threadEntry->tid_os = folly::getOSThreadID();
-
-      // if we're adding a thread entry
-      // we need to increment the list count
-      // even if the entry is reused
-      threadEntryList->count++;
-
-      threadEntry->meta = &meta;
+      threadEntry = meta.allocateNewThreadEntry();
       int ret = pthread_setspecific(key, threadEntry);
       checkPosixError(ret, "pthread_setspecific failed");
     }
