@@ -14,4 +14,4 @@
  * limitations under the License.
  */
 
-#include <folly/coro/detail/Traits.h>
+#include <folly/coro/detail/Traits.h> // @shim
