@@ -150,7 +150,7 @@ AsyncIoUringSocket::ReadSqe::ReadSqe(AsyncIoUringSocket* parent)
 AsyncIoUringSocket::~AsyncIoUringSocket() {
   VLOG(3) << "~AsyncIoUringSocket() " << this;
 
-  // this is a bit unnecesary if we are already closed, but proper state
+  // this is a bit unnecessary if we are already closed, but proper state
   // tracking is coming later and will be easier to handle then
   closeNow();
 
@@ -277,9 +277,11 @@ void AsyncIoUringSocket::connect(
     // IP_BIND_ADDRESS_NO_PORT forces the OS to find a unique port relying
     // on only the local tuple. This limits the range of available ephemeral
     // ports.  Using the IP_BIND_ADDRESS_NO_PORT delays assigning a port until
-    // connect expanding the available port range.
+    // connect expanding the available port range, unless
+    // setBindAddressNoPort() is called.
     if (bindAddr.getPort() == 0) {
-      if (setSockOpt(IPPROTO_IP, IP_BIND_ADDRESS_NO_PORT, &one, sizeof(one))) {
+      if (bindAddressNoPort_ &&
+          setSockOpt(IPPROTO_IP, IP_BIND_ADDRESS_NO_PORT, &one, sizeof(one))) {
         auto errnoCopy = errno;
         callback->connectErr(AsyncSocketException(
             AsyncSocketException::NOT_OPEN,
@@ -540,9 +542,8 @@ void AsyncIoUringSocket::ReadSqe::setReadCallback(
       << " inflight=" << inFlight() << " good_=" << parent_->good()
       << " submitNow=" << submitNow;
 
-  if (callback == readCallback_) {
-    // copied from AsyncSocket
-    VLOG(9) << "cb the same";
+  if (callback == readCallback_ && (!submitNow || inFlight())) {
+    VLOG(9) << "cb the same, skipping";
     return;
   }
   setReadCbCount_++;
@@ -613,14 +614,13 @@ void AsyncIoUringSocket::ReadSqe::callback(const io_uring_cqe* cqe) noexcept {
           << " has_buffer=" << !!(flags & IORING_CQE_F_BUFFER)
           << " bytes_received=" << bytesReceived_;
   DestructorGuard dg(this);
-  auto buffer_guard = makeGuard([&, bp = lastUsedBufferProvider_] {
-    if (flags & IORING_CQE_F_BUFFER) {
-      DCHECK(bp);
-      if (bp) {
-        bp->unusedBuf(flags >> 16);
-      }
-    }
+  bool hasMore = (flags & IORING_CQE_F_BUF_MORE) != 0;
+  auto buffer_guard = makeGuard([&] {
+    CHECK(!(flags & IORING_CQE_F_BUFFER))
+        << "Buffer guard invoked but IORING_CQE_F_BUFFER is set! " << "flags=0x"
+        << std::hex << flags << " res=" << std::dec << res;
   });
+
   if (!readCallback_) {
     if (res == -ENOBUFS || res == -ECANCELED) {
       // ignore
@@ -632,7 +632,7 @@ void AsyncIoUringSocket::ReadSqe::callback(const io_uring_cqe* cqe) noexcept {
     } else if (res > 0 && lastUsedBufferProvider_) {
       // must take the buffer
       appendReadData(
-          lastUsedBufferProvider_->getIoBuf(flags >> 16, res),
+          lastUsedBufferProvider_->getIoBuf(flags >> 16, res, hasMore),
           queuedReceivedData_);
       buffer_guard.dismiss();
     }
@@ -653,7 +653,7 @@ void AsyncIoUringSocket::ReadSqe::callback(const io_uring_cqe* cqe) noexcept {
       }
     } else if (res < 0) {
       // assume ECANCELED is not an unrecoverable error state, but we do still
-      // have to propogate to the callback as they presumably called the cancel.
+      // have to propagate to the callback as they presumably called the cancel.
       auto callback = readCallback_;
       if (parent_ && res != -ECANCELED) {
         readCallback_ = nullptr;
@@ -687,9 +687,13 @@ void AsyncIoUringSocket::ReadSqe::callback(const io_uring_cqe* cqe) noexcept {
         const io_uring_zcrx_cqe* rcqe = (io_uring_zcrx_cqe*)(cqe + 1);
         auto pool = parent_->backend_->zcBufferPool();
         sendReadBuf(pool->getIoBuf(cqe, rcqe), queuedReceivedData_);
+        buffer_guard.dismiss();
       } else if (lastUsedBufferProvider_) {
+        auto bufId = flags >> 16;
+        VLOG(9) << "Processing buffer completion: bufId=" << bufId
+                << " res=" << res << " hasMore=" << hasMore;
         sendReadBuf(
-            lastUsedBufferProvider_->getIoBuf(flags >> 16, res),
+            lastUsedBufferProvider_->getIoBuf(bufId, res, hasMore),
             queuedReceivedData_);
         buffer_guard.dismiss();
       } else {
@@ -1279,7 +1283,7 @@ void AsyncIoUringSocket::processWriteQueue() noexcept {
     shutdownWriteNow();
     return;
   }
-  if (state_ != State::Established) {
+  if (state_ != State::Established && !connecting()) {
     failAllWrites();
     return;
   }
@@ -1799,7 +1803,7 @@ int AsyncIoUringSocket::setSockOpt(
   return ::setsockopt(fd_.toFd(), level, optname, optval, optsize);
 }
 
-bool AsyncIoUringSocket::getTFOSucceded() const {
+bool AsyncIoUringSocket::getTFOSucceeded() const {
   return detail::tfo_succeeded(fd_);
 }
 
