@@ -97,7 +97,8 @@ IoUringProvidedBufferRing::IoUringProvidedBufferRing(
           options.count,
           options.bufferShift,
           options.ringSizeShift,
-          options.useHugePages) {
+          options.useHugePages),
+      useIncremental_(options.useIncrementalBuffers) {
   if (options.count > std::numeric_limits<uint16_t>::max()) {
     throw std::runtime_error("too many buffers");
   }
@@ -105,12 +106,15 @@ IoUringProvidedBufferRing::IoUringProvidedBufferRing(
     throw std::runtime_error("not enough buffers");
   }
 
-  ioBufCallbacks_.assign(
-      (options.count + (sizeof(void*) - 1)) / sizeof(void*), this);
-
   initialRegister();
 
-  gottenBuffers_ += options.count;
+  bufferStates_ = std::make_unique<BufferState[]>(options.count);
+  for (uint16_t i = 0; i < options.count; i++) {
+    bufferStates_[i].bufId = i;
+    bufferStates_[i].parent = this;
+    bufferStates_[i].offset = 0;
+  }
+
   for (size_t i = 0; i < options.count; i++) {
     returnBuffer(i);
   }
@@ -130,11 +134,6 @@ void IoUringProvidedBufferRing::enobuf() noexcept {
   VLOG_EVERY_N(1, 500) << "enobuf";
 }
 
-void IoUringProvidedBufferRing::unusedBuf(uint16_t i) noexcept {
-  gottenBuffers_++;
-  returnBuffer(i);
-}
-
 void IoUringProvidedBufferRing::destroy() noexcept {
   std::unique_lock lock{mutex_};
   ::io_uring_unregister_buf_ring(ioRingPtr_, gid());
@@ -146,29 +145,66 @@ void IoUringProvidedBufferRing::destroy() noexcept {
   delayedDestroy(remaining);
 }
 
+void IoUringProvidedBufferRing::returnBuffer(uint16_t i) noexcept {
+  std::unique_lock lock{mutex_};
+  if (FOLLY_UNLIKELY(wantsShutdown_)) {
+    auto refs = --shutdownReferences_;
+    lock.unlock();
+    delayedDestroy(refs);
+    return;
+  }
+
+  if (useIncremental_) {
+    bufferStates_[i].offset = 0;
+    bufferStates_[i].refCount.store(1);
+  }
+
+  uint16_t this_idx = static_cast<uint16_t>(ringReturnedBuffers_++);
+  uint16_t next_tail = this_idx + 1;
+
+  __u64 addr = (__u64)buffer_.buffer(i);
+  auto* r = buffer_.ringBuf(this_idx);
+  r->addr = addr;
+  r->len = buffer_.sizePerBuffer();
+  r->bid = i;
+
+  if (tryPublish(this_idx, next_tail)) {
+    enobuf_.store(false, std::memory_order_relaxed);
+  }
+  VLOG(9) << "returnBuffer(" << i << ")@" << this_idx;
+}
+
 std::unique_ptr<IOBuf> IoUringProvidedBufferRing::getIoBuf(
-    uint16_t i, size_t length) noexcept {
+    uint16_t i, size_t length, bool hasMore) noexcept {
   std::unique_ptr<IOBuf> ret;
   DCHECK(!wantsShutdown_);
+  DCHECK_LT(i, buffer_.bufferCount())
+      << "Buffer index " << i << " exceeds buffer count "
+      << buffer_.bufferCount();
 
-  // use a weird convention: userData = ioBufCallbacks_.data() + i
-  // ioBufCallbacks_ is just a list of the same pointer, to this
-  // so we don't need to malloc anything
   auto free_fn = [](void*, void* userData) {
-    size_t pprov = (size_t)userData & ~((size_t)(sizeof(void*) - 1));
-    IoUringProvidedBufferRing* prov = *(IoUringProvidedBufferRing**)pprov;
-    uint16_t idx = (size_t)userData - (size_t)prov->ioBufCallbacks_.data();
-    prov->returnBuffer(idx);
+    auto* bufferState = static_cast<BufferState*>(userData);
+    IoUringProvidedBufferRing* parent = bufferState->parent;
+    uint16_t bufId = bufferState->bufId;
+
+    parent->decBufferState(bufId);
   };
 
-  ret = IOBuf::takeOwnership(
-      (void*)getData(i),
-      sizePerBuffer_,
-      length,
-      free_fn,
-      (void*)(((size_t)ioBufCallbacks_.data()) + i));
+  if (useIncremental_) {
+    auto* bufferStart = getData(i);
+    unsigned int currentOffset = bufferStates_[i].offset;
+    auto* dataPtr = bufferStart + currentOffset;
+    BufferState* info = &bufferStates_[i];
+    ret = IOBuf::takeOwnership((void*)dataPtr, length, length, free_fn, info);
+  } else {
+    BufferState* info = &bufferStates_[i];
+    ret = IOBuf::takeOwnership(
+        (void*)getData(i), sizePerBuffer_, length, free_fn, info);
+  }
+
   ret->markExternallySharedOne();
-  gottenBuffers_++;
+  incBufferState(i, hasMore, length);
+
   return ret;
 }
 
@@ -179,7 +215,12 @@ void IoUringProvidedBufferRing::initialRegister() {
   reg.ring_entries = buffer_.ringCount();
   reg.bgid = gid();
 
-  int ret = ::io_uring_register_buf_ring(ioRingPtr_, &reg, 0);
+  int flags = 0;
+  if (useIncremental_) {
+    flags |= IOU_PBUF_RING_INC;
+  }
+
+  int ret = ::io_uring_register_buf_ring(ioRingPtr_, &reg, flags);
 
   if (ret) {
     LOG(ERROR) << folly::to<std::string>(
@@ -206,28 +247,35 @@ void IoUringProvidedBufferRing::delayedDestroy(uint64_t refs) noexcept {
   }
 }
 
-void IoUringProvidedBufferRing::returnBuffer(uint16_t i) noexcept {
-  std::unique_lock lock{mutex_};
-  if (FOLLY_UNLIKELY(wantsShutdown_)) {
-    auto refs = --shutdownReferences_;
-    lock.unlock();
-    delayedDestroy(refs);
+void IoUringProvidedBufferRing::incBufferState(
+    uint16_t bufId, bool hasMore, unsigned int bytesConsumed) noexcept {
+  gottenBuffers_++;
+
+  if (useIncremental_ && hasMore) {
+    uint16_t oldRefCount = bufferStates_[bufId].refCount.fetch_add(1);
+    bufferStates_[bufId].offset += bytesConsumed;
+
+    VLOG(9) << "Buffer " << bufId << " refcount=" << oldRefCount + 1
+            << " offset=" << bufferStates_[bufId].offset
+            << " hasMore=" << hasMore << " bytesConsumed=" << bytesConsumed;
+  }
+
+  // No need to handle regular buffers, since it is never really incremented
+  // beyond the original assingment of 1 instead of 0.
+}
+
+void IoUringProvidedBufferRing::decBufferState(uint16_t bufId) noexcept {
+  returnedBuffers_++;
+  if (!useIncremental_ || wantsShutdown_) {
+    returnBuffer(bufId);
     return;
   }
-
-  uint16_t this_idx = static_cast<uint16_t>(returnedBuffers_++);
-  uint16_t next_tail = this_idx + 1;
-
-  __u64 addr = (__u64)buffer_.buffer(i);
-  auto* r = buffer_.ringBuf(this_idx);
-  r->addr = addr;
-  r->len = buffer_.sizePerBuffer();
-  r->bid = i;
-
-  if (tryPublish(this_idx, next_tail)) {
-    enobuf_.store(false, std::memory_order_relaxed);
+  uint16_t oldRefCount = bufferStates_[bufId].refCount.fetch_sub(1);
+  VLOG(9) << "Buffer " << bufId << " refcount=" << oldRefCount - 1
+          << " (decremented)";
+  if (oldRefCount == 1) {
+    returnBuffer(bufId);
   }
-  VLOG(9) << "returnBuffer(" << i << ")@" << this_idx;
 }
 
 } // namespace folly
