@@ -31,6 +31,8 @@
 #include <folly/io/async/EventBase.h>
 #include <folly/io/async/IoUringBackend.h>
 #include <folly/io/async/IoUringEvent.h>
+#include <folly/io/async/test/AsyncSocketTest.h>
+#include <folly/io/async/test/AsyncSocketTest2.h>
 #include <folly/portability/GTest.h>
 #include <folly/system/Shell.h>
 #include <folly/test/SocketAddressTestHelper.h>
@@ -641,7 +643,7 @@ TEST_P(AsyncIoUringSocketTest, FastOpen) {
     EXPECT_EQ(
         "hello", conn.callback->waitFor(5).via(base.get()).getVia(base.get()));
     if (!had_fastopen) {
-      EXPECT_FALSE(conn.client->transport->getTFOSucceded());
+      EXPECT_FALSE(conn.client->transport->getTFOSucceeded());
     }
   }
   {
@@ -649,11 +651,75 @@ TEST_P(AsyncIoUringSocketTest, FastOpen) {
     EXPECT_EQ(
         "hello", conn.callback->waitFor(5).via(base.get()).getVia(base.get()));
     if (can_fastopen) {
-      EXPECT_TRUE(conn.client->transport->getTFOSucceded());
+      EXPECT_TRUE(conn.client->transport->getTFOSucceeded());
     }
   }
 }
 
+TEST_P(AsyncIoUringSocketTest, BindAddressNoPort) {
+  EventBase eventBase;
+  test::TestServer server(true);
+
+  // When setBindAddressNoPort is disabled, verifies that a port is assigned
+  // before the connect call
+  AsyncIoUringSocket::UniquePtr socket(new AsyncIoUringSocket(base.get()));
+  socket->setBindAddressNoPort(false);
+  SocketAddress bindAddr("127.0.0.1", 0);
+  test::TestPortAssignmentCallback callback;
+  socket->connect(
+      &callback,
+      server.getAddress(),
+      std::chrono::milliseconds(30),
+      emptySocketOptionMap,
+      bindAddr);
+  eventBase.loop();
+  EXPECT_NE(callback.assignedPort, 0);
+  socket->close();
+}
+
+TEST_P(AsyncIoUringSocketTest, ReadCallbackSetDuringConnect) {
+  MAYBE_SKIP();
+
+  struct ReadCB : public AsyncReader::ReadCallback {
+    void getReadBuffer(void** bufReturn, size_t* lenReturn) override {
+      *bufReturn = buff;
+      *lenReturn = sizeof(buff);
+    }
+
+    void readDataAvailable(size_t len) noexcept override {
+      prom.setValue(std::string{buff, len});
+    }
+
+    void readEOF() noexcept override {}
+    void readErr(const AsyncSocketException& ex) noexcept override {
+      prom.setException(ex);
+    }
+
+    bool isBufferMovable() noexcept override { return false; }
+
+    Promise<std::string> prom;
+    char buff[1024];
+  };
+
+  ReadCB readCB;
+  AsyncIoUringSocket::UniquePtr socket(
+      new AsyncIoUringSocket(base.get(), ioUringSocketOptions()));
+
+  socket->connect(this, serverAddress);
+  socket->setReadCB(&readCB);
+  auto fd =
+      fdPromise.getFuture().within(kTimeout).via(base.get()).getVia(base.get());
+  fdPromise = {};
+  auto server = AsyncSocket::newSocket(base.get(), fd);
+  server->write(&nullWriteCallback, "hello", 5);
+  auto result =
+      readCB.prom.getSemiFuture()
+          .within(kTimeout)
+          .via(base.get())
+          .getVia(base.get());
+  EXPECT_EQ("hello", result);
+}
+
 class AsyncIoUringSocketTestAll : public AsyncIoUringSocketTest {};
 
 TEST_P(AsyncIoUringSocketTestAll, WriteChain2) {
