@@ -16,6 +16,7 @@
 
 #include <gtest/gtest.h>
 
+#include <folly/coro/Traits.h>
 #include <folly/result/gtest_helpers.h>
 
 #if FOLLY_HAS_RESULT
@@ -144,7 +145,7 @@ TEST(Result, awaitStoppedResult) {
     LOG(FATAL) << "not reached";
   };
   auto outerFn = [&]() -> result<> {
-    co_await innerFn();
+    co_await or_unwind(innerFn());
     LOG(FATAL) << "not reached";
   };
   auto r = outerFn();
@@ -155,12 +156,12 @@ TEST(Result, awaitStoppedResult) {
 RESULT_CO_TEST(Result, CTAD) {
   result ri = 42;
   static_assert(std::is_same_v<result<int>, decltype(ri)>);
-  EXPECT_EQ(42, co_await std::move(ri));
+  EXPECT_EQ(42, co_await or_unwind(std::move(ri)));
 
   result rsp = std::make_unique<std::string>("foo");
   static_assert(
       std::is_same_v<result<std::unique_ptr<std::string>>, decltype(rsp)>);
-  EXPECT_EQ(std::string{"foo"}, *(co_await std::cref(rsp)));
+  EXPECT_EQ(std::string{"foo"}, *(co_await or_unwind(std::as_const(rsp))));
 }
 
 TEST(Result, movable) {
@@ -175,7 +176,9 @@ TEST(Result, movable) {
 
 TEST(Result, refCopiable) {
   auto intPtr = std::make_unique<int>(1337);
-  result<std::unique_ptr<int>&> mIntPtrRef1 = std::ref(intPtr);
+  result mIntPtrRef1 = std::ref(intPtr);
+  static_assert(
+      std::is_same_v<result<std::unique_ptr<int>&>, decltype(mIntPtrRef1)>);
   auto mIntPtrRef2 = mIntPtrRef1.copy();
   *(mIntPtrRef2.value_or_throw()) += 1;
   EXPECT_EQ(1338, *mIntPtrRef1.value_or_throw());
@@ -216,6 +219,68 @@ TEST(Result, fromNonValue) {
   }
 }
 
+RESULT_CO_TEST(Result, ofLvalueReferenceWrapper) {
+  int n = 3;
+  // Yes, you can declare `result<reference_wrapper<V>>`.  This is one way to
+  // mutate values through `const result<Ref>` (the other being `result<V*>`).
+  {
+    result<std::reference_wrapper<int>> r = std::ref(n);
+    // The `.get()` is here to show that a ref-wrapper is being returned.
+    EXPECT_EQ(3, (co_await or_unwind(std::move(r))).get());
+  }
+  // To store `result<V&>`, you can use CTAD
+  {
+    result r = std::ref(n);
+    static_assert(std::is_same_v<result<int&>, decltype(r)>);
+    EXPECT_EQ(3, co_await or_unwind(std::move(r)));
+  }
+}
+
+RESULT_CO_TEST(Result, ofRvalueReferenceWrapper) {
+  // Yes, you can declare `result<rvalue_reference_wrapper<V>>`.
+  {
+    int n = 3;
+    result<rvalue_reference_wrapper<int>> r = rref(std::move(n));
+    // The `.get()` is here to show that a ref-wrapper is being returned.
+    EXPECT_EQ(3, (co_await or_unwind(std::move(r))).get());
+  }
+  // To store `result<V&&>`, you can use CTAD
+  {
+    int n = 3;
+    result r = rref(std::move(n));
+    static_assert(std::is_same_v<result<int&&>, decltype(r)>);
+    EXPECT_EQ(3, co_await or_unwind(std::move(r)));
+  }
+}
+
+RESULT_CO_TEST(Result, forbidUnsafeCopyOfResultRef) {
+  int n = 42;
+
+  result rc = std::cref(n);
+  static_assert(std::is_same_v<result<const int&>, decltype(rc)>);
+  { // Safe copies of ref -- `rc` has `const` inside, cannot be discarded
+    result rc2 = rc.copy();
+    EXPECT_EQ(42, (co_await or_unwind(rc2)));
+    result rc3 = std::as_const(rc).copy();
+    EXPECT_EQ(42, (co_await or_unwind(rc3)));
+  }
+  static_assert(requires { rc.copy(); });
+  static_assert(requires { std::as_const(rc).copy(); });
+
+  result<int&> r = std::ref(n);
+  { // Safe copy of ref -- `r` has no `const` to discard
+    result r2 = r.copy();
+    EXPECT_EQ(42, (co_await or_unwind(r2)));
+  }
+  // Unsafe: copying `const result<int&>` would discard the outer `const`
+  //   result r3 = std::as_const(r).copy();
+  static_assert(requires { r.copy(); });
+  [](const auto& cr) {
+    // This `requires` won't even compile outside a template context.
+    static_assert(!requires { cr.copy(); });
+  }(r);
+}
+
 // Check `?.value_or_throw()` and `co_await ?` return types for various ways of
 // accessing `result<V&>` and `result<V&&>`.
 //
@@ -228,49 +293,38 @@ TEST(Result, fromNonValue) {
 //
 //   result<int&> rn = n();
 //   rn.value_or_throw() -> int&
-//   co_await std::move(rn) -> int&
-//   co_await std::ref(rn) -> int&
+//   co_await or_unwind(rn) -> int&
+//   co_await or_unwind(std::move(rn)) -> int&&
 //
-// However, both of the following COULD potentially return `const int&`.
+//   !! `result.md` explains why these return ref-to-const !!
 //
-//   std::as_const(rn).value_or_throw() -> int& OR const int&
-//   co_await std::cref(rn) -> int& OR const int&
-//
-// We chose NOT to add `const` for two reasons:
-//   - A `const` instance of `std::reference_wrapper` gives non-const access to
-//     the underlying reference, and we advertise `result` reference support as
-//     "syntax sugar for reference wrappers".
-//   - `result<int&>` is like a non-null `result<int*>`.  Adding `const` to the
-//     latter does not, and definitely should not, make the pointer `const`.
+//   std::as_const(rn).value_or_throw() -> const int&
+//   co_await or_unwind(std::as_const(rn)) -> const int&
 template <typename T>
 void checkAwaitResumeTypeForRefResult() {
-  // 'co_await' cannot be used in an unevaluated context
-  auto fakeCoAwait = [](auto&& r) -> decltype(auto) {
-    return operator co_await(static_cast<decltype(r)&&>(r)).await_resume();
-  };
-
   // `result`s with r-value refs must be r-value qualified for access, since
   // `folly::rref` models a "use-once" / "destructive-access" reference.
-  static_assert( // `co_await`
-      std::is_same_v<
-          T&&,
-          decltype(fakeCoAwait(std::move(FOLLY_DECLVAL(result<T&&>&&))))>);
+  using AwRR = decltype(or_unwind(std::move(FOLLY_DECLVAL(result<T&&>&&))));
+  static_assert( // co_await or_unwind(resFn())
+      std::is_same_v<T&&, coro::await_result_t<AwRR>>);
   static_assert( // `value_or_throw` follows `co_await`
       std::is_same_v<
           T&&,
           decltype(FOLLY_DECLVAL(result<T&&>&&).value_or_throw())>);
 
   // `co_await`ing a `result` with an l-value ref always returns the ref
-  static_assert( // `co_await std::ref(r)`
-      std::is_same_v<
-          T&,
-          decltype(fakeCoAwait(std::ref(FOLLY_DECLVAL(result<T&>&))))>);
-  static_assert( // `co_await std::cref(r)`
-      std::is_same_v<
-          T&,
-          decltype(fakeCoAwait(std::cref(FOLLY_DECLVAL(result<T&>&))))>);
-  static_assert( // `co_await std::move(r)`
-      std::is_same_v<T&, decltype(fakeCoAwait(FOLLY_DECLVAL(result<T&>&&)))>);
+
+  // `co_await or_unwind(r)`
+  using AwLL = decltype(or_unwind(FOLLY_DECLVAL(result<T&>&)));
+  static_assert(std::is_same_v<T&, coro::await_result_t<AwLL>>);
+
+  // `co_await or_unwind(std::as_const(r))`
+  using AwCLL = decltype(or_unwind(FOLLY_DECLVAL(const result<T&>&)));
+  static_assert(std::is_same_v<const T&, coro::await_result_t<AwCLL>>);
+
+  // `co_await or_unwind(std::move(r))`
+  using AwLR = decltype(or_unwind(FOLLY_DECLVAL(result<T&>&&)));
+  static_assert(std::is_same_v<T&, coro::await_result_t<AwLR>>);
 
   // `value_or_throw()` behaves like `co_await` for l-value ref `result`s
   static_assert(
@@ -279,7 +333,7 @@ void checkAwaitResumeTypeForRefResult() {
           decltype(FOLLY_DECLVAL(result<T&>&).value_or_throw())>);
   static_assert(
       std::is_same_v< // `std::as_const(r).value_or_throw()`
-          T&, // fn docblock explains why this is NOT `const`
+          const T&,
           decltype(FOLLY_DECLVAL(const result<T&>&).value_or_throw())>);
   static_assert(
       std::is_same_v< // `std::move(r).value_or_throw()`
@@ -297,31 +351,32 @@ RESULT_CO_TEST(Result, fromRefWrapperAndRefAccess) {
   T t2 = std::make_unique<int>(567);
   {
     result<T&> rLref = std::ref(t1);
-    EXPECT_EQ(321, *(co_await rLref.copy()));
+    EXPECT_EQ(321, *(co_await or_unwind(rLref.copy())));
     EXPECT_EQ(321, *rLref.value_or_throw());
-    *(co_await std::ref(rLref)) += 1;
+    *(co_await or_unwind(rLref)) += 1;
     EXPECT_EQ(322, *t1);
-    (co_await std::move(rLref)) = std::move(t2);
+    (co_await or_unwind(std::move(rLref))) = std::move(t2);
     EXPECT_EQ(567, *t1);
   }
 
   {
     result<const T&> rCref = std::cref(t1);
-    EXPECT_EQ(567, *(co_await rCref.copy()));
+    EXPECT_EQ(567, *(co_await or_unwind(rCref.copy())));
     EXPECT_EQ(567, *rCref.value_or_throw());
-    *(co_await std::cref(rCref)) += 1; // can change the int, not the unique_ptr
+    *(co_await or_unwind(std::as_const(rCref))) +=
+        1; // can change the int, not the unique_ptr
     EXPECT_EQ(568, *t1);
 
     EXPECT_TRUE(t2 == nullptr); // was moved out above
     t2 = std::make_unique<int>(42);
     rCref = std::cref(t2); // assignment uses the implict ctor
-    EXPECT_EQ(42, *(co_await rCref.copy()));
+    EXPECT_EQ(42, *(co_await or_unwind(rCref.copy())));
   }
 
   {
     result<T&&> rRref1 = rref(std::move(t1));
     result<T&&> rRref2 = rref(std::move(rRref1).value_or_throw());
-    t2 = std::move(co_await std::move(rRref2));
+    t2 = std::move(co_await or_unwind(std::move(rRref2)));
     EXPECT_EQ(568, *t2);
   }
 }
@@ -342,7 +397,7 @@ RESULT_CO_TEST(Result, movableContexts) {
       auto fivePtr = std::make_unique<int>(5);
       co_return fivePtr;
     };
-    EXPECT_EQ(5, *(co_await fn()));
+    EXPECT_EQ(5, *(co_await or_unwind(fn())));
   }
   {
     auto fn = []() -> result<std::unique_ptr<int>> {
@@ -374,9 +429,9 @@ RESULT_CO_TEST(Result, copySmallTrivialUnderlying) {
   };
   Smol s{13, 21};
   result<Smol> m{s};
-  EXPECT_EQ(s, co_await std::ref(m));
-  EXPECT_EQ(s, co_await std::move(m));
-  EXPECT_EQ(s, co_await result<Smol>{s});
+  EXPECT_EQ(s, co_await or_unwind(m));
+  EXPECT_EQ(s, co_await or_unwind(std::move(m)));
+  EXPECT_EQ(s, co_await or_unwind(result<Smol>{s}));
 }
 
 RESULT_CO_TEST(Result, simpleConversion) {
@@ -394,55 +449,55 @@ RESULT_CO_TEST(Result, simpleConversion) {
       // magic for string literals, but that's not the main point of this test.
       return result<const char*>{"resFn"};
     };
-    EXPECT_EQ(std::string("resFn"), co_await resFn());
+    EXPECT_EQ(std::string("resFn"), co_await or_unwind(resFn()));
 
     auto coResFn = []() -> result<std::string> {
       return result<const char*>{"coResFn"};
     };
-    EXPECT_EQ(std::string("coResFn"), co_await coResFn());
+    EXPECT_EQ(std::string("coResFn"), co_await or_unwind(coResFn()));
   }
 
   // Copying conversion with `&` binding
   {
     result<int> rn{rCrefN1}; // ctor converts from ref wrapper
     result<float> rf{rn}; // ctor converts from value
-    EXPECT_EQ(1000, co_await std::move(rn));
-    EXPECT_EQ(1000, co_await std::move(rf));
+    EXPECT_EQ(1000, co_await or_unwind(std::move(rn)));
+    EXPECT_EQ(1000, co_await or_unwind(std::move(rf)));
 
     rn = rCrefN2; // assignment converts from ref wrapper
     rf = rn; // assignment converts from value
-    EXPECT_EQ(2000, co_await std::move(rn));
-    EXPECT_EQ(2000, co_await std::move(rf));
+    EXPECT_EQ(2000, co_await or_unwind(std::move(rn)));
+    EXPECT_EQ(2000, co_await or_unwind(std::move(rf)));
   }
 
   // Copying conversion with `const &` binding
   {
     result<int> rn{std::as_const(rCrefN1)}; // ctor converts from ref wrapper
-    EXPECT_EQ(1000, co_await std::ref(rn));
+    EXPECT_EQ(1000, co_await or_unwind(rn));
 
     result<float> rf{std::as_const(rn)}; // ctor converts from value
-    EXPECT_EQ(1000, co_await std::ref(rf));
+    EXPECT_EQ(1000, co_await or_unwind(rf));
 
     rn = std::as_const(rCrefN2); // assignment converts from ref wrapper
-    EXPECT_EQ(2000, co_await std::ref(rn));
+    EXPECT_EQ(2000, co_await or_unwind(rn));
 
     rf = std::as_const(rn); // assignment converts from value
-    EXPECT_EQ(2000, co_await std::ref(rf));
+    EXPECT_EQ(2000, co_await or_unwind(rf));
   }
 
   // (Possibly) move conversion with `&&` binding
   {
     result<int> rn{std::move(rCrefN1)}; // ctor converts from ref wrapper
-    EXPECT_EQ(1000, co_await std::cref(rn));
+    EXPECT_EQ(1000, co_await or_unwind(std::as_const(rn)));
 
     result<float> rf{std::move(rn)}; // ctor converts from value
-    EXPECT_EQ(1000, co_await std::cref(rf));
+    EXPECT_EQ(1000, co_await or_unwind(std::as_const(rf)));
 
     rn = std::move(rCrefN2); // assignment converts from ref wrapper
-    EXPECT_EQ(2000, co_await std::cref(rn));
+    EXPECT_EQ(2000, co_await or_unwind(std::as_const(rn)));
 
     rf = std::move(rn); // assignment converts from value
-    EXPECT_EQ(2000, co_await std::cref(rf));
+    EXPECT_EQ(2000, co_await or_unwind(std::as_const(rf)));
   }
 }
 
@@ -475,7 +530,7 @@ RESULT_CO_TEST(Result, fallibleConversion) {
     vs.emplace_back(ConversionCanFail{3});
     int sum = 0;
     for (result<int> r : vs) {
-      sum += co_await std::move(r);
+      sum += co_await or_unwind(std::move(r));
     }
     EXPECT_EQ(11, sum);
   }
@@ -484,7 +539,7 @@ RESULT_CO_TEST(Result, fallibleConversion) {
   {
     result<ConversionCanFail> convOk5 = ConversionCanFail{.v_ = 5};
     result<int> ok5{convOk5};
-    EXPECT_EQ(5, co_await std::move(ok5));
+    EXPECT_EQ(5, co_await or_unwind(std::move(ok5)));
 
     result<ConversionCanFail> convFail0 = ConversionCanFail{.v_ = 0};
     result<int> fail0{convFail0};
@@ -495,7 +550,7 @@ RESULT_CO_TEST(Result, fallibleConversion) {
   {
     const result<ConversionCanFail> convOk5 = ConversionCanFail{.v_ = 5};
     result<int> ok5{convOk5};
-    EXPECT_EQ(5, co_await std::move(ok5));
+    EXPECT_EQ(5, co_await or_unwind(std::move(ok5)));
 
     const result<ConversionCanFail> convFail0 = ConversionCanFail{.v_ = 0};
     result<int> fail0{convFail0};
@@ -506,7 +561,7 @@ RESULT_CO_TEST(Result, fallibleConversion) {
   {
     result<ConversionCanFail> convOk5 = ConversionCanFail{.v_ = 5};
     result<int> ok5{std::move(convOk5)};
-    EXPECT_EQ(5, co_await std::move(ok5));
+    EXPECT_EQ(5, co_await or_unwind(std::move(ok5)));
 
     result<ConversionCanFail> convFail0 = ConversionCanFail{.v_ = 0};
     result<int> fail0{std::move(convFail0)};
@@ -541,7 +596,9 @@ void test_bad_empty_result(auto bad) {
   EXPECT_FALSE(errRes == bad);
   EXPECT_TRUE(errRes != bad);
   EXPECT_TRUE(bad == bad);
-  auto awaitsBad = [&]() -> result<> { co_await std::cref(bad); };
+  auto awaitsBad = [&]() -> result<> {
+    co_await or_unwind(std::as_const(bad));
+  };
   if constexpr (!kIsDebug) {
     auto res = awaitsBad();
     EXPECT_TRUE(get_exception<detail::empty_result_error>(res));
@@ -740,7 +797,7 @@ RESULT_CO_TEST(Result, returnImplicitCtor) {
   auto returnImplicitCtor = []() -> result<std::pair<int, int>> {
     co_return {3, 4};
   };
-  EXPECT_EQ(4, (co_await returnImplicitCtor()).second);
+  EXPECT_EQ(4, (co_await or_unwind(returnImplicitCtor())).second);
 }
 
 TEST(Result, checkCustomError) {
@@ -748,7 +805,7 @@ TEST(Result, checkCustomError) {
     return non_value_result{MyError{"kthxbai"}};
   };
   auto shortCircuitOnError = [&]() -> result<std::string> {
-    co_await returnsMyError();
+    co_await or_unwind(returnsMyError());
     co_return "not reached";
   };
   auto r = shortCircuitOnError();
@@ -764,35 +821,39 @@ TEST(Result, isExceptionBoundary) {
   EXPECT_EQ(std::string("catch me"), get_exception<MyError>(r)->what());
 }
 
-// `result_owning_awaitable` propagating a value
 RESULT_CO_TEST(Result, awaitValue) {
   auto returnsValue = []() -> result<uint8_t> { return 129; };
   // Test both EXPECT and CO_ASSERT
-  EXPECT_EQ(129, co_await returnsValue());
-  CO_ASSERT_EQ(129, co_await returnsValue());
+  EXPECT_EQ(129, co_await or_unwind(returnsValue()));
+  CO_ASSERT_EQ(129, co_await or_unwind(returnsValue()));
 }
 
-// `result_ref_awaitable` propagating a value
 RESULT_CO_TEST(Result, awaitRef) {
   // Use a non-copiable type to show that `co_await` doesn't copy.
   result<std::unique_ptr<int>> var = std::make_unique<int>(17);
-  auto& varRef = *(co_await std::ref(var));
+  auto& varRef = *(co_await or_unwind(var));
   EXPECT_EQ(17, varRef);
   varRef = 42;
-  EXPECT_EQ(42, *(co_await std::cref(var)));
-  EXPECT_EQ(42, *(co_await std::move(var)));
-  // Test that `var` is now in the moved-out state
+  EXPECT_EQ(42, *(co_await or_unwind(std::as_const(var))));
+
+  EXPECT_EQ(42, *(co_await or_unwind(std::move(var))));
+  // `var` is not moved out yet! We just took a reference above.
   // NOLINTNEXTLINE(bugprone-use-after-move)
-  EXPECT_FALSE(co_await std::cref(var));
+  EXPECT_EQ(42, *(co_await or_unwind(var)));
+
+  // NOLINTNEXTLINE(bugprone-use-after-move)
+  auto ptr = co_await or_unwind(std::move(var));
+  EXPECT_EQ(42, *ptr);
+  // Now, `var` is moved out & null.
+  // NOLINTNEXTLINE(bugprone-use-after-move)
+  EXPECT_FALSE(co_await or_unwind(std::as_const(var)));
 }
 
 TEST(Result, awaitError) {
   for (auto& r :
-       {// `error_awaitable`
-        []() -> result<> { co_await non_value_result{MyError{"eep"}}; }(),
-        // `result_owning_awaitable` propagating an error
+       {[]() -> result<> { co_await non_value_result{MyError{"eep"}}; }(),
         []() -> result<> {
-          co_await result<int>{non_value_result{MyError{"eep"}}};
+          co_await or_unwind(result<int>{non_value_result{MyError{"eep"}}});
         }()}) {
     EXPECT_EQ(std::string("eep"), get_exception<MyError>(r)->what());
   }
@@ -801,8 +862,7 @@ TEST(Result, awaitError) {
 TEST(Result, awaitRefError) {
   auto resultErrFn = []() -> result<> {
     result<std::unique_ptr<int>> resultErr{non_value_result{MyError{"e"}}};
-    // `result_ref_awaitable` propagating error
-    co_await std::cref(resultErr);
+    co_await or_unwind(std::as_const(resultErr));
   };
   auto res = resultErrFn();
   EXPECT_TRUE(get_exception<MyError>(res));
@@ -815,7 +875,7 @@ TEST(Result, containsRValueRef) {
   };
   auto middle = [&]() -> result<std::string&&> { co_return inner(); };
   auto outer = [&]() -> result<std::string&&> {
-    co_return rref(co_await middle());
+    co_return rref(co_await or_unwind(middle()));
   };
 
   auto&& moveMeRef = outer().value_or_throw();
@@ -835,7 +895,7 @@ class ResultTest : public testing::Test {
 
 RESULT_CO_TEST_F(ResultTest, check_TEST_F) {
   result<int> r{getMember()};
-  EXPECT_EQ(co_await std::ref(r), 42);
+  EXPECT_EQ(co_await or_unwind(r), 42);
 }
 
 TEST(Result, catch_all_returns_result_error) {
